/*******************************************************************************
**3456789 123456789 123456789 123456789 123456789 123456789 123456789 123456789
**      10        20        30        40        50        60        70        80
**
** project:
**    "Bling it up - make it sexy!"
**
** file:
**    cairo-rendering.c
**
** author:
**    Mirco "MacSlow" Mueller <macslow@bangang.de>
**
** copyright (C) Mirco Mueller, february 2007, placed under the terms of the GPL
**
*******************************************************************************/

#include <math.h>
#include <time.h>
#include <X11/Xlib.h>
#include <cairo-xlib.h>

#include "event-handler.h"
#include "glitz-helper.h"
#include "cairo-rendering.h"

/* that's so nasty! */
extern glitz_drawable_format_t*	g_pDrawableFormat;
extern glitz_drawable_t*	g_pDrawable;
extern glitz_format_t*		g_pFormat;
extern gboolean			g_bUseGlitz;
extern GTimer*			g_pTimer;
extern gint			g_iRefreshRate;
cairo_surface_t* 		g_pSurface[SURFACE_MAX_ELEMENTS];
RsvgHandle*			g_pSvgHandle	  = NULL;
RsvgDimensionData		g_dimension	  = {0, 0, 0.0f, 0.0f};
double				g_fSliderPosition = 0.0f;
Stage				g_iStage	  = STAGE_START;

cairo_surface_t*
update_surface (Surface		   whichSurface,
		gint		   iWindowWidth,
		gint		   iWindowHeight,
		RsvgHandle*	   pSvgHandle,
		RsvgDimensionData* pDimension)
{
	cairo_t*	 pCairoContext = NULL;
	cairo_surface_t* pSurface      = NULL;

	/* create surface */
	if (g_bUseGlitz)
	{
		glitz_surface_attributes_t attribs	  = {TRUE};
		glitz_drawable_t*	   pGlitzDrawable = NULL;
		glitz_surface_t*	   pGlitzSurface  = NULL;

		/* try to get a FBO-drawable */
		pGlitzDrawable = glitz_create_drawable (g_pDrawable,
							g_pDrawableFormat,
							iWindowWidth,
							iWindowHeight);

		if (!pGlitzDrawable)
		{
			/* try to get a pbuffer-drawable */
			pGlitzDrawable = glitz_create_pbuffer_drawable (
							g_pDrawable,
							g_pDrawableFormat,
							iWindowWidth,
							iWindowHeight);
		}

		if (!pGlitzDrawable)
		{
			g_print ("update_surface(): Argl, no glitz-drawable!\n");
			return NULL;
		}

		pGlitzSurface = glitz_surface_create (
					pGlitzDrawable,
					g_pFormat,
					iWindowWidth,
					iWindowHeight,
					GLITZ_SURFACE_UNNORMALIZED_MASK,
					&attribs);

		pSurface = cairo_glitz_surface_create (pGlitzSurface);
	}
	else
	{
		Pixmap pixmap;
		pixmap = XCreatePixmap (GDK_DISPLAY (),
					GDK_ROOT_WINDOW (),
					(unsigned int) iWindowWidth,
					(unsigned int) iWindowHeight,
					(unsigned int) 32);

		pSurface = cairo_xlib_surface_create (
				GDK_DISPLAY (),
				pixmap,
				GDK_VISUAL_XVISUAL (
					gdk_screen_get_rgba_visual (
						gdk_screen_get_default ())),
				iWindowWidth,
				iWindowHeight);
	}

	/* create a temporary context */
	pCairoContext = cairo_create (pSurface);

	/* adjust context to window size and SVG-dimension, clear it */
	cairo_scale (pCairoContext,
		     (double) iWindowWidth  / (double) pDimension->width,
		     (double) iWindowHeight / (double) pDimension->height);
	cairo_set_operator (pCairoContext, CAIRO_OPERATOR_OVER);
	cairo_set_source_rgba (pCairoContext, 0.0f, 0.0f, 0.0f, 0.0f);
	cairo_paint (pCairoContext);

	/* draw selected layer onto surface */
	switch (whichSurface)
	{
		case SURFACE_GLOW :
			rsvg_handle_render_cairo_sub (pSvgHandle,
						      pCairoContext,
						      "#Slider-Glow");
		break;

		case SURFACE_TEXT :
			rsvg_handle_render_cairo_sub (pSvgHandle,
						      pCairoContext,
						      "#Slider-Text");
		break;

		case SURFACE_BUTTON :
			rsvg_handle_render_cairo_sub (pSvgHandle,
						      pCairoContext,
						      "#Slider-Button");
		break;

		case SURFACE_CLOCK :
			rsvg_handle_render_cairo_sub (pSvgHandle,
						      pCairoContext,
						      "#Analog-Clock");
		break;

		case SURFACE_BACKGROUND_LOGO :
			rsvg_handle_render_cairo_sub (pSvgHandle,
						      pCairoContext,
						      "#Background-Logo");
		break;

		case SURFACE_BACKGROUND_SLIDER :
			rsvg_handle_render_cairo_sub (pSvgHandle,
						      pCairoContext,
						      "#Background-Slider");
			rsvg_handle_render_cairo_sub (pSvgHandle,
						      pCairoContext,
						      "#Slider-Button");
			rsvg_handle_render_cairo_sub (pSvgHandle,
						      pCairoContext,
						      "#Slider-Text");
		break;

		case SURFACE_BACKGROUND :
			rsvg_handle_render_cairo_sub (pSvgHandle,
						      pCairoContext,
						      "#Background");
		break;

		case SURFACE_BACKGROUND_FULL :
			rsvg_handle_render_cairo_sub (pSvgHandle,
						      pCairoContext,
						      "#Background");
			rsvg_handle_render_cairo_sub (pSvgHandle,
						      pCairoContext,
						      "#Background-Logo");
			rsvg_handle_render_cairo_sub (pSvgHandle,
						      pCairoContext,
						      "#Background-Slider");
		break;

		default :
		break;
	}

	/* get rid of temporary context */
	cairo_destroy (pCairoContext);

	return pSurface;
}

void
update_surface_buffers (gint iWindowWidth,
			gint iWindowHeight)

{
	/* load SVG from disk if not already loaded */
	if (!g_pSvgHandle)
	{
		GError* pError = NULL;

		g_pSvgHandle = rsvg_handle_new_from_file ("./resources/ui-thing.svg",
							  &pError);
		rsvg_handle_get_dimensions (g_pSvgHandle, &g_dimension);
	}

	cairo_surface_destroy (g_pSurface[SURFACE_GLOW]);
	g_pSurface[SURFACE_GLOW] = update_surface (SURFACE_GLOW,
						   iWindowWidth,
						   iWindowHeight,
						   g_pSvgHandle,
						   &g_dimension);

	cairo_surface_destroy (g_pSurface[SURFACE_TEXT]);
	g_pSurface[SURFACE_TEXT] = update_surface (SURFACE_TEXT,
						   iWindowWidth,
						   iWindowHeight,
						   g_pSvgHandle,
						   &g_dimension);

	cairo_surface_destroy (g_pSurface[SURFACE_BUTTON]);
	g_pSurface[SURFACE_BUTTON] = update_surface (SURFACE_BUTTON,
						     iWindowWidth,
						     iWindowHeight,
						     g_pSvgHandle,
						     &g_dimension);

	cairo_surface_destroy (g_pSurface[SURFACE_CLOCK]);
	g_pSurface[SURFACE_CLOCK] = update_surface (SURFACE_CLOCK,
						    iWindowWidth,
						    iWindowHeight,
						    g_pSvgHandle,
						    &g_dimension);

	cairo_surface_destroy (g_pSurface[SURFACE_BACKGROUND_LOGO]);
	g_pSurface[SURFACE_BACKGROUND_LOGO] = update_surface (
							SURFACE_BACKGROUND_LOGO,
							iWindowWidth,
							iWindowHeight,
							g_pSvgHandle,
							&g_dimension);

	cairo_surface_destroy (g_pSurface[SURFACE_BACKGROUND_SLIDER]);
	g_pSurface[SURFACE_BACKGROUND_SLIDER] = update_surface (
							SURFACE_BACKGROUND_SLIDER,
							iWindowWidth,
							iWindowHeight,
							g_pSvgHandle,
							&g_dimension);

	cairo_surface_destroy (g_pSurface[SURFACE_BACKGROUND]);
	g_pSurface[SURFACE_BACKGROUND] = update_surface (SURFACE_BACKGROUND,
							 iWindowWidth,
							 iWindowHeight,
							 g_pSvgHandle,
							 &g_dimension);

	cairo_surface_destroy (g_pSurface[SURFACE_BACKGROUND_FULL]);
	g_pSurface[SURFACE_BACKGROUND_FULL] = update_surface (
							SURFACE_BACKGROUND_FULL,
							iWindowWidth,
							iWindowHeight,
							g_pSvgHandle,
							&g_dimension);
}

void
draw_surface_with_clip (cairo_t* pContext,
			cairo_surface_t* pSurface,
			double fAlpha,
			double fOffsetX,
			double fOffsetY,
			double fClipX,
			double fClipY,
			double fClipWidth,
			double fClipHeight)
{
	if (fAlpha == 0.0f)
		return;

	cairo_rectangle (pContext, fClipX, fClipY, fClipWidth, fClipHeight);
	cairo_clip (pContext);
	cairo_set_source_surface (pContext,
				  pSurface,
				  fOffsetX,
				  fOffsetY);

	if (fAlpha < 1.0f)
		cairo_paint_with_alpha (pContext, fAlpha);
	else
		cairo_paint (pContext);

	cairo_reset_clip (pContext);
}

void
render (cairo_t* pCairoContext,
	gint	 iWindowWidth,
	gint	 iWindowHeight)
{
	static double	fClockAlpha	       = 0.0f;
	static double	fGlowPosition	       = 0.0f;
	static double	fFadeIn		       = 0.0f;
	static double	fFadeOut	       = 0.0f;
	static double	fStep		       = 0.03f;
	static int	iFrames		       = 0;
	static double	fLastTimeStamp	       = 0.0f;
	static double	fCurrentTimeStamp      = 0.0f;
	static double	fLastFullSecond	       = 0.0f;
	unsigned long	ulMilliSeconds	       = 0;
	double		fFullSecond	       = 0.0f;
	double		fFactor		       = 0.0f;
	static time_t	timeOfDay;
	struct tm*	pTime;
	int		iSecond;
	int		iMinute;
	int		iHour;
	static double	fAngleSecond	       = 0.0f;
	static double	fAngleMinute	       = 0.0f;
	static double	fAngleHour	       = 0.0f;
	static gboolean bAnimateMinute	       = FALSE;
	double		fValue		       = 0.0;

	/* determine time and get hand-angles for minute and hour */
	time (&timeOfDay);
	pTime	= localtime (&timeOfDay);
	iSecond	= pTime->tm_sec;
	iMinute	= pTime->tm_min;
	iHour	= pTime->tm_hour;

	if (!bAnimateMinute)
		fAngleMinute = (double) iMinute * 6.0f;

	fAngleHour = (iSecond + 60 * iMinute + 3600 * (iHour % 12)) *
		     360.0 /
		     43200.0f;

	/* clear the "canvas" */
	cairo_scale (pCairoContext, 1.0f, 1.0f);
	cairo_set_operator (pCairoContext, CAIRO_OPERATOR_OVER);
	cairo_set_source_rgba (pCairoContext, 0.0f, 0.0f, 0.0f, 1.0f);
	cairo_paint (pCairoContext);

	switch (g_iStage)
	{
		case STAGE_START:
			fValue = sin (G_PI / 2.0f * fFadeIn);

			/* paste the background into the window */
			draw_surface_with_clip (pCairoContext,
						g_pSurface[SURFACE_BACKGROUND],
						fValue,
						0.0f,
						0.0f,
						0.0f,
						0.0f,
						480.0f,
						700.0f);

			/* slide the background-logo into the window */
			draw_surface_with_clip (
				pCairoContext,
				g_pSurface[SURFACE_BACKGROUND_LOGO],
				fValue,
				0.0f,
				-167.0f * (1.0f - fValue),
				0.0f,
				0.0f,
				480.0f,
				167.0f);

			/* slide the background-logo into the window */
			draw_surface_with_clip (
				pCairoContext,
				g_pSurface[SURFACE_BACKGROUND_SLIDER],
				fValue,
				0.0f,
				167.0f * (1.0f - fValue),
				0.0f,
				533.0f,
				480.0f,
				167.0f);

			fFadeIn += fStep;
			fFadeOut = fFadeIn;

			if (fFadeIn > 1.0f)
			{
				g_iStage = STAGE_RUNNING;
				fFadeIn = 1.0f;
				fFadeOut = 1.0f;
			}
		break;

		case STAGE_RUNNING:
			/* paste the background into the window */
			draw_surface_with_clip (
				pCairoContext,
				g_pSurface[SURFACE_BACKGROUND_FULL],
				1.0f,
				0.0f,
				0.0f,
				0.0f,
				0.0f,
				480.0f,
				700.0f);

			/* get the current time-stamp */
			fCurrentTimeStamp = g_timer_elapsed (g_pTimer,
							     &ulMilliSeconds);
			ulMilliSeconds /= 1000;
			fFullSecond = fCurrentTimeStamp - fLastFullSecond;

			/* paste the analog-clock face into the window, note the
			** alpha-value! */
			if (fClockAlpha > 0.0f)
			{
				double fOffsetX = (1.0f - fClockAlpha) * 480.0f;

				/* draw clock face */
				draw_surface_with_clip (
					pCairoContext,
					g_pSurface[SURFACE_CLOCK],
					fClockAlpha,
					fOffsetX,
					0.0f,
					0.0f,
					178.0f,
					480.0f,
					350.0f);

				/* determine animation-factor/value for
				** clock-hands */
				fFactor = powf ((float) iFrames /
						(float) g_iRefreshRate *
						0.875f *
						3.4f,
						3.0f) *
					  sinf ((float) iFrames /
						(float) g_iRefreshRate *
						0.875f *
						G_PI) *
						0.1f;

				/* draw hour-hands shadow */
				cairo_set_line_cap (pCairoContext,
						    CAIRO_LINE_CAP_ROUND);
				cairo_set_line_width (pCairoContext, 9.0f);
				cairo_save (pCairoContext);
				cairo_translate (pCairoContext, 2.5f, 2.5f);
				cairo_save (pCairoContext);
				cairo_translate (pCairoContext,
						 240.0f + fOffsetX,
						 350.0f);
				cairo_rotate (pCairoContext,
					      G_PI / 180.0f * fAngleHour);
				cairo_set_source_rgba (pCairoContext,
						       0.0f,
						       0.0f,
						       0.0f,
						       0.3f * fClockAlpha);
				cairo_move_to (pCairoContext, 0.0f, 0.0f);
				cairo_line_to (pCairoContext, 0.0f, -85.0f);
				cairo_stroke (pCairoContext);
				cairo_restore (pCairoContext);
				cairo_restore (pCairoContext);

				/* draw hour-hand */
				cairo_save (pCairoContext);
				cairo_translate (pCairoContext,
						 240.0f + fOffsetX,
						 350.0f);
				cairo_rotate (pCairoContext,
					      G_PI / 180.0f * fAngleHour);
				cairo_set_source_rgba (pCairoContext,
						       1.0f,
						       1.0f,
						       1.0f,
						       1.0f * fClockAlpha);
				cairo_move_to (pCairoContext, 0.0f, 0.0f);
				cairo_line_to (pCairoContext, 0.0f, -85.0f);
				cairo_stroke (pCairoContext);
				cairo_restore (pCairoContext);

				/* draw minute-hands shadow */
				cairo_set_line_width (pCairoContext, 6.0f);
				cairo_save (pCairoContext);
				cairo_translate (pCairoContext, 4.0f, 4.0f);
				cairo_save (pCairoContext);
				cairo_translate (pCairoContext,
						 240.0f + fOffsetX,
						 350.0f);
				if (bAnimateMinute)
				{
					cairo_rotate (
						pCairoContext,
						G_PI / 180.0f *
						(fAngleMinute + fFactor * 6.0f));
					if (fFactor >= 0.95f)
					{
						bAnimateMinute = FALSE;
						fAngleMinute =
							(double) iMinute * 6.0f;
					}
				}
				else
					cairo_rotate (
						pCairoContext,
						G_PI / 180.0f * fAngleMinute);

				cairo_set_source_rgba (pCairoContext,
						       0.0f,
						       0.0f,
						       0.0f,
						       0.3f * fClockAlpha);
				cairo_move_to (pCairoContext, 0.0f, 0.0f);
				cairo_line_to (pCairoContext, 0.0f, -110.0f);
				cairo_stroke (pCairoContext);
				cairo_restore (pCairoContext);
				cairo_restore (pCairoContext);

				/* draw minute-hand */
				cairo_save (pCairoContext);
				cairo_translate (pCairoContext,
						 240.0f + fOffsetX,
						 350.0f);
				if (bAnimateMinute)
				{
					cairo_rotate (
						pCairoContext,
						G_PI / 180.0f *
						(fAngleMinute + fFactor * 6.0f));
					if (fFactor >= 0.95f)
					{
						bAnimateMinute = FALSE;
						fAngleMinute =
							(double) iMinute * 6.0f;
					}
				}
				else
					cairo_rotate (
						pCairoContext,
						G_PI / 180.0f * fAngleMinute);

				cairo_set_source_rgba (pCairoContext,
						       1.0f,
						       1.0f,
						       1.0f,
						       1.0f * fClockAlpha);
				cairo_move_to (pCairoContext, 0.0f, 0.0f);
				cairo_line_to (pCairoContext, 0.0f, -110.0f);
				cairo_stroke (pCairoContext);
				cairo_restore (pCairoContext);

				/* draw second-hands shadow */
				cairo_set_line_width (pCairoContext, 3.0f);
				cairo_save (pCairoContext);
				cairo_translate (pCairoContext, 5.5f, 5.5f);
				cairo_save (pCairoContext);
				cairo_translate (pCairoContext,
						 240.0f + fOffsetX,
						 350.0f);
				cairo_rotate (pCairoContext,
					      G_PI / 180.0f *
					      (fAngleSecond + fFactor * 6.0f));

				cairo_set_source_rgba (pCairoContext,
						       0.0f,
						       0.0f,
						       0.0f,
						       0.3f * fClockAlpha);
				cairo_move_to (pCairoContext, 0.0f, 0.0f);
				cairo_line_to (pCairoContext, 0.0f, -135.0f);
				cairo_stroke (pCairoContext);
				cairo_arc (pCairoContext,
					   0.0f,
					   0.0f,
					   3.5f,
					   0.0f * G_PI / 180.0f,
					   360.0f * G_PI / 180.0f);
				cairo_fill (pCairoContext);
				cairo_restore (pCairoContext);
				cairo_restore (pCairoContext);

				/* draw second-hand */
				cairo_save (pCairoContext);
				cairo_translate (pCairoContext,
						 240.0f + fOffsetX,
						 350.0f);
				cairo_rotate (pCairoContext,
					      G_PI / 180.0f *
					      (fAngleSecond + fFactor * 6.0f));
				cairo_set_line_cap (pCairoContext,
						    CAIRO_LINE_CAP_ROUND);
				cairo_set_source_rgba (pCairoContext,
						       0.8f,
						       0.1f,
						       0.1f,
						       1.0f * fClockAlpha);
				cairo_move_to (pCairoContext, 0.0f, 0.0f);
				cairo_line_to (pCairoContext, 0.0f, -135.0f);
				cairo_stroke (pCairoContext);
				cairo_arc (pCairoContext,
					   0.0f,
					   0.0f,
					   3.5f,
					   0.0f * G_PI / 180.0f,
					   360.0f * G_PI / 180.0f);
				cairo_fill (pCairoContext);
				cairo_restore (pCairoContext);
			}

			/* paste the text of the slider into the window */
			draw_surface_with_clip (
				pCairoContext,
				g_pSurface[SURFACE_TEXT],
				0.4f,
				0.0f,
				0.0f,
				155.0f + g_fSliderPosition * 286.0f,
				602.0f,
				286.0f * (1.0f - g_fSliderPosition),
				37.0f);
			cairo_push_group (pCairoContext);
			draw_surface_with_clip (
				pCairoContext,
				g_pSurface[SURFACE_GLOW],
				1.0f,
				fGlowPosition * 400.0f,
				0.0f,
				155.0f + g_fSliderPosition * 286.0f,
				602.0f,
				286.0f * (1.0f - g_fSliderPosition),
				37.0f);
			cairo_set_operator (pCairoContext,
					    CAIRO_OPERATOR_DEST_IN);
			draw_surface_with_clip (pCairoContext,
						g_pSurface[SURFACE_TEXT],
						1.0f,
						0.0f,
						0.0f,
						155.0f + g_fSliderPosition * 286.0f,
						602.0f,
						286.0f * (1.0f - g_fSliderPosition),
						37.0f);
			cairo_pop_group_to_source (pCairoContext);
			cairo_rectangle (pCairoContext,
					 155.0f,
					 602.0f,
					 286.0f,
					 37.0f);
			cairo_clip (pCairoContext);
			cairo_paint (pCairoContext);
			cairo_reset_clip (pCairoContext);

			/* paste the slider-button into the window, note the
			** position */
			draw_surface_with_clip (pCairoContext,
						g_pSurface[SURFACE_BUTTON],
						1.0f,
						294.0f * g_fSliderPosition,
						0.0f,
						33.0f,
						573.0f,
						414.0f,
						90.0f);

			fGlowPosition += fStep - 0.01;

			if (fGlowPosition > 1.0f)
				fGlowPosition = 0.0f;
		break;

		case STAGE_END:
			fValue = sin (G_PI / 2.0f * fFadeOut);

			if (g_fSliderPosition > 0.0f)
			{
				/* paste the background into the window */
				draw_surface_with_clip (pCairoContext,
							g_pSurface[SURFACE_BACKGROUND_FULL],
							1.0f,
							0.0f,
							0.0f,
							0.0f,
							0.0f,
							480.0f,
							700.0f);

				/* get the current time-stamp */
				fCurrentTimeStamp = g_timer_elapsed (g_pTimer, &ulMilliSeconds);
				ulMilliSeconds /= 1000;
				fFullSecond = fCurrentTimeStamp - fLastFullSecond;

				/* paste the analog-clock face into the window, note the alpha-value! */
				if (fClockAlpha > 0.0f)
				{
					double fOffsetX = (1.0f - fClockAlpha) * 480.0f;

					/* draw clock face */
					draw_surface_with_clip (pCairoContext,
								g_pSurface[SURFACE_CLOCK],
								fClockAlpha,
								fOffsetX,
								0.0f,
								0.0f,
								178.0f,
								480.0f,
								350.0f);

					/* determine animation-factor/value for clock-hands */
					fFactor = powf ((float) iFrames /
							(float) g_iRefreshRate *
							0.875f *
							3.4f,
							3.0f) *
						  sinf ((float) iFrames /
							(float) g_iRefreshRate *
							0.875f *
							G_PI) *
							0.1f;

					/* draw hour-hands shadow */
					cairo_set_line_cap (pCairoContext, CAIRO_LINE_CAP_ROUND);
					cairo_set_line_width (pCairoContext, 9.0f);
					cairo_save (pCairoContext);
					cairo_translate (pCairoContext, 2.5f, 2.5f);
					cairo_save (pCairoContext);
					cairo_translate (pCairoContext, 240.0f + fOffsetX, 350.0f);
					cairo_rotate (pCairoContext, G_PI / 180.0f * fAngleHour);
					cairo_set_source_rgba (pCairoContext,
							       0.0f,
							       0.0f,
							       0.0f,
							       0.3f * fClockAlpha);
					cairo_move_to (pCairoContext, 0.0f, 0.0f);
					cairo_line_to (pCairoContext, 0.0f, -85.0f);
					cairo_stroke (pCairoContext);
					cairo_restore (pCairoContext);
					cairo_restore (pCairoContext);

					/* draw hour-hand */
					cairo_save (pCairoContext);
					cairo_translate (pCairoContext, 240.0f + fOffsetX, 350.0f);
					cairo_rotate (pCairoContext, G_PI / 180.0f * fAngleHour);
					cairo_set_source_rgba (pCairoContext,
							       1.0f,
							       1.0f,
							       1.0f,
							       1.0f * fClockAlpha);
					cairo_move_to (pCairoContext, 0.0f, 0.0f);
					cairo_line_to (pCairoContext, 0.0f, -85.0f);
					cairo_stroke (pCairoContext);
					cairo_restore (pCairoContext);

					/* draw minute-hands shadow */
					cairo_set_line_width (pCairoContext, 6.0f);
					cairo_save (pCairoContext);
					cairo_translate (pCairoContext, 4.0f, 4.0f);
					cairo_save (pCairoContext);
					cairo_translate (pCairoContext, 240.0f + fOffsetX, 350.0f);
					if (bAnimateMinute)
					{
						cairo_rotate (pCairoContext,
							      G_PI / 180.0f *
							      (fAngleMinute + fFactor * 6.0f));
						if (fFactor >= 0.95f)
						{
							bAnimateMinute = FALSE;
							fAngleMinute = (double) iMinute * 6.0f;
						}
					}
					else
						cairo_rotate (pCairoContext,
							      G_PI / 180.0f * fAngleMinute);

					cairo_set_source_rgba (pCairoContext,
							       0.0f,
							       0.0f,
							       0.0f,
							       0.3f * fClockAlpha);
					cairo_move_to (pCairoContext, 0.0f, 0.0f);
					cairo_line_to (pCairoContext, 0.0f, -110.0f);
					cairo_stroke (pCairoContext);
					cairo_restore (pCairoContext);
					cairo_restore (pCairoContext);

					/* draw minute-hand */
					cairo_save (pCairoContext);
					cairo_translate (pCairoContext, 240.0f + fOffsetX, 350.0f);
					if (bAnimateMinute)
					{
						cairo_rotate (pCairoContext,
							      G_PI / 180.0f *
							      (fAngleMinute + fFactor * 6.0f));
						if (fFactor >= 0.95f)
						{
							bAnimateMinute = FALSE;
							fAngleMinute = (double) iMinute * 6.0f;
						}
					}
					else
						cairo_rotate (pCairoContext,
							      G_PI / 180.0f * fAngleMinute);

					cairo_set_source_rgba (pCairoContext,
							       1.0f,
							       1.0f,
							       1.0f,
							       1.0f * fClockAlpha);
					cairo_move_to (pCairoContext, 0.0f, 0.0f);
					cairo_line_to (pCairoContext, 0.0f, -110.0f);
					cairo_stroke (pCairoContext);
					cairo_restore (pCairoContext);

					/* draw second-hands shadow */
					cairo_set_line_width (pCairoContext, 3.0f);
					cairo_save (pCairoContext);
					cairo_translate (pCairoContext, 5.5f, 5.5f);
					cairo_save (pCairoContext);
					cairo_translate (pCairoContext, 240.0f + fOffsetX, 350.0f);
					cairo_rotate (pCairoContext,
						      G_PI / 180.0f *
						      (fAngleSecond + fFactor * 6.0f));

					cairo_set_source_rgba (pCairoContext,
							       0.0f,
							       0.0f,
							       0.0f,
							       0.3f * fClockAlpha);
					cairo_move_to (pCairoContext, 0.0f, 0.0f);
					cairo_line_to (pCairoContext, 0.0f, -135.0f);
					cairo_stroke (pCairoContext);
					cairo_arc (pCairoContext,
						   0.0f,
						   0.0f,
						   3.5f,
						   0.0f * G_PI / 180.0f,
						   360.0f * G_PI / 180.0f);
					cairo_fill (pCairoContext);
					cairo_restore (pCairoContext);
					cairo_restore (pCairoContext);

					/* draw second-hand */
					cairo_save (pCairoContext);
					cairo_translate (pCairoContext, 240.0f + fOffsetX, 350.0f);
					cairo_rotate (pCairoContext,
						      G_PI / 180.0f *
						      (fAngleSecond + fFactor * 6.0f));
					cairo_set_line_cap (pCairoContext, CAIRO_LINE_CAP_ROUND);
					cairo_set_source_rgba (pCairoContext,
							       0.8f,
							       0.1f,
							       0.1f,
							       1.0f * fClockAlpha);
					cairo_move_to (pCairoContext, 0.0f, 0.0f);
					cairo_line_to (pCairoContext, 0.0f, -135.0f);
					cairo_stroke (pCairoContext);
					cairo_arc (pCairoContext,
						   0.0f,
						   0.0f,
						   3.5f,
						   0.0f * G_PI / 180.0f,
						   360.0f * G_PI / 180.0f);
					cairo_fill (pCairoContext);
					cairo_restore (pCairoContext);
				}


				/* paste the text of the slider into the window */
				draw_surface_with_clip (pCairoContext,
							g_pSurface[SURFACE_TEXT],
							0.4f,
							0.0f,
							0.0f,
							155.0f + g_fSliderPosition * 286.0f,
							602.0f,
							286.0f * (1.0f - g_fSliderPosition),
							36.0f);
				cairo_push_group (pCairoContext);
				draw_surface_with_clip (pCairoContext,
							g_pSurface[SURFACE_GLOW],
							1.0f,
							fGlowPosition * 400.0f,
							0.0f,
							155.0f + g_fSliderPosition * 286.0f,
							602.0f,
							286.0f * (1.0f - g_fSliderPosition),
							37.0f);
				cairo_set_operator (pCairoContext, CAIRO_OPERATOR_DEST_IN);
				draw_surface_with_clip (pCairoContext,
							g_pSurface[SURFACE_TEXT],
							1.0f,
							0.0f,
							0.0f,
							155.0f + g_fSliderPosition * 286.0f,
							602.0f,
							286.0f * (1.0f - g_fSliderPosition),
							36.0f);
				cairo_pop_group_to_source (pCairoContext);
				cairo_paint (pCairoContext);

				/* paste the slider-button into the window, note the position */
				draw_surface_with_clip (pCairoContext,
							g_pSurface[SURFACE_BUTTON],
							1.0f,
							294.0f * g_fSliderPosition,
							0.0f,
							33.0f,
							573.0f,
							414.0f,
							90.0f);

				fGlowPosition += fStep - 0.01;

				if (fGlowPosition > 1.0f)
					fGlowPosition = 0.0f;

				g_fSliderPosition -= fStep;
			}
			else
			{
				/* paste the background into the window */
				draw_surface_with_clip (pCairoContext,
							g_pSurface[SURFACE_BACKGROUND],
							fValue,
							0.0f,
							0.0f,
							0.0f,
							0.0f,
							480.0f,
							700.0f);

				/* slide the background-logo into the window */
				draw_surface_with_clip (pCairoContext,
							g_pSurface[SURFACE_BACKGROUND_LOGO],
							fValue,
							0.0f,
							-167.0f * (1.0f - fValue),
							0.0f,
							0.0f,
							480.0f,
							167.0f);

				/* slide the background-logo into the window */
				draw_surface_with_clip (pCairoContext,
							g_pSurface[SURFACE_BACKGROUND_SLIDER],
							fValue,
							0.0f,
							167.0f * (1.0f - fValue),
							0.0f,
							533.0f,
							480.0f,
							167.0f);

				fFadeOut -= fStep;
			}

			if (fFadeOut < 0.02f)
				delete_handler (NULL, NULL, NULL);
		break;

		default:
		break;
	}

	/* make the clock "fade in" based on the slider position */
	if (g_fSliderPosition >= 0.5f)
		fClockAlpha = (g_fSliderPosition - 0.5f) * 2.0f;
	else
		fClockAlpha = 0.0f;

	/* take care of the clock-hands anim-vars */
	if (fFullSecond < 1.0f)
		iFrames++;
	else
	{
		iFrames = 0;
		fLastFullSecond = fCurrentTimeStamp;

		fAngleSecond = (double) iSecond * 6.0f;

		if (fAngleSecond == 354.0f)
			bAnimateMinute = TRUE;

		if (fAngleSecond == 360.0f)
			fAngleSecond = 0.0f;

		if (fAngleMinute == 360.0f)
			fAngleMinute = 0.0f;
	}
	fLastTimeStamp = fCurrentTimeStamp;

	/* if double-buffered swap the drawables buffers */
	if (g_bUseGlitz && g_pDrawableFormat->doublebuffer)
		glitz_drawable_swap_buffers (g_pDrawable);
}

void
cleanup (void)
{
	cairo_surface_destroy (g_pSurface[SURFACE_GLOW]);
	cairo_surface_destroy (g_pSurface[SURFACE_TEXT]);
	cairo_surface_destroy (g_pSurface[SURFACE_BUTTON]);
	cairo_surface_destroy (g_pSurface[SURFACE_CLOCK]);
	cairo_surface_destroy (g_pSurface[SURFACE_BACKGROUND]);
	cairo_surface_destroy (g_pSurface[SURFACE_BACKGROUND_LOGO]);
	cairo_surface_destroy (g_pSurface[SURFACE_BACKGROUND_SLIDER]);
	cairo_surface_destroy (g_pSurface[SURFACE_BACKGROUND_FULL]);
}
