/*******************************************************************************
**3456789 123456789 123456789 123456789 123456789 123456789 123456789 123456789
**      10        20        30        40        50        60        70        80
**
** project:
**    "Bling it up - make it sexy!"
**
** file:
**    event-handler.c
**
** author:
**    Mirco "MacSlow" Mueller <macslow@bangang.de>
**
** copyright (C) Mirco Mueller, february 2007, placed under the terms of the GPL
**
*******************************************************************************/

#include <gdk/gdkkeysyms.h>

#include "event-handler.h"
#include "glitz-helper.h"
#include "cairo-rendering.h"

extern double			g_fSliderPosition;
extern Stage			g_iStage;
extern glitz_drawable_format_t*	g_pDrawableFormat;
extern glitz_drawable_t*	g_pDrawable;
extern glitz_format_t*		g_pFormat;
extern gboolean			g_bUseGlitz;
gboolean			g_bSliderIsDragged = FALSE;
double				g_fLastX	   = 0.0f;

gboolean
delete_handler (GtkWidget* pWidget,
		GdkEvent*  pEvent,
		gpointer   data)
{
	cleanup ();
	gtk_main_quit ();

	return FALSE;
}

gboolean
fake_delete_handler (GtkWidget* pWidget,
		     GdkEvent*  pEvent,
		     gpointer   data)
{
	g_iStage = STAGE_END;

	return TRUE;
}

gboolean
configure_handler (GtkWidget*	      pWidget,
		   GdkEventConfigure* pEvent,
		   gpointer	      data)
{
	static gint iOldWidth  = 0;
	static gint iOldHeight = 0;
	gint	    iWidth     = 0;
	gint	    iHeight    = 0;

	iWidth	= pEvent->width;
	iHeight	= pEvent->height;

	if (iOldWidth != iWidth || iOldHeight != iHeight)
		update_surface_buffers (iWidth, iHeight);

	iOldWidth  = iWidth;
	iOldHeight = iHeight;

	return FALSE;
}

gboolean
expose_handler (GtkWidget*	pWidget,
		GdkEventExpose*	pEvent,
		gpointer	data)
{
	gint	 iWidth   = 1;
	gint	 iHeight  = 1;
	cairo_t* pContext = NULL;

	gtk_window_get_size (GTK_WINDOW (pWidget), &iWidth, &iHeight);

	if (g_bUseGlitz)
		pContext = create_glitz_context (pWidget->window,
						 g_pDrawableFormat,
						 g_pDrawable,
						 g_pFormat,
						 iWidth,
						 iHeight);
	else
		pContext = gdk_cairo_create (pWidget->window);

	if (!pContext)
		return FALSE;

	render (pContext, iWidth, iHeight);

	cairo_destroy (pContext);

	return FALSE;
}

gboolean
key_press_handler (GtkWidget*	pWidget,
		   GdkEventKey*	pEvent,
		   gpointer	data)
{
	if (pEvent->type == GDK_KEY_PRESS)
	{
		switch (pEvent->keyval)
		{
			/* exit program */
			case GDK_q :
			case GDK_Escape :
				g_iStage = STAGE_END;
			break;
		}
	}

	return FALSE;
}

gboolean
slider_is_hit (double fX, double fY)
{
	if (fX < 33.0f + 114.0f + 294.0f * g_fSliderPosition &&
	    fX > 33.0f + 294.0f * g_fSliderPosition &&
	    fY < 660.0f &&
	    fY > 577.0f)
	{
		g_fLastX = fX;
		return TRUE;
	}
	else
		return FALSE;
}

gboolean
button_handler (GtkWidget*	pWidget,
		GdkEventButton*	pEvent,
		gpointer	data)
{
	gboolean bSliderIsHit = slider_is_hit (pEvent->x, pEvent->y);

	/* mark that the slider is being dragged */
	if (pEvent->type == GDK_BUTTON_PRESS && pEvent->button == 1)
		if (bSliderIsHit && !g_bSliderIsDragged)
			g_bSliderIsDragged = TRUE;

	/* stop dragging the slider */
	if (pEvent->type == GDK_BUTTON_RELEASE && pEvent->button == 1)
		if (g_bSliderIsDragged)
			g_bSliderIsDragged = FALSE;

	/* move the window around */
	if ((pEvent->type == GDK_BUTTON_PRESS) && (pEvent->button == 1))
		if (!g_bSliderIsDragged)
			gtk_window_begin_move_drag (
				GTK_WINDOW (gtk_widget_get_toplevel (pWidget)),
				pEvent->button,
				pEvent->x_root,
				pEvent->y_root,
				pEvent->time);

	return TRUE;
}

gboolean
motion_handler (GtkWidget*	pWidget,
		GdkEventMotion*	pEvent,
		gpointer	data)
{
	double fNewPosition = g_fSliderPosition;

	if (g_bSliderIsDragged &&
	    g_fSliderPosition <= 1.0f &&
	    g_fSliderPosition >= 0.0f )
	{
		fNewPosition += (-g_fLastX + pEvent->x) / 294.0f;

		if (fNewPosition > 1.0f)
			g_fSliderPosition = 1.0f;
		if (fNewPosition < 0.0f)
			g_fSliderPosition = 0.0f;
		else if (fNewPosition < 1.0f && fNewPosition > 0.0f)
			g_fSliderPosition = fNewPosition;

		g_fLastX = pEvent->x;
	}

	return TRUE;
}

gboolean
draw_handler (GtkWidget* pWidget)
{
	/* force expose/refresh */
	gtk_widget_queue_draw (pWidget);

	return TRUE;
}
