/*******************************************************************************
**3456789 123456789 123456789 123456789 123456789 123456789 123456789 123456789
**      10        20        30        40        50        60        70        80
**
** project:
**    "Bling it up - make it sexy!"
**
** file:
**    glitz-helper.c
**
** author:
**    Mirco "MacSlow" Mueller <macslow@bangang.de>
**
** copyright (C) Mirco Mueller, february 2007, placed under the terms of the GPL
**
*******************************************************************************/

#include "glitz-helper.h"

glitz_drawable_format_t*
get_glitz_drawable_format (GtkWidget* pWidget,
			   gint	      iScreen)
{
	glitz_drawable_format_t	 formatTemplate;
	glitz_drawable_format_t* pCurrentFormat = NULL;
	glitz_drawable_format_t* pResultFormat = NULL;
	unsigned long		 ulMask = 0;
	XVisualInfo*		 pVisualInfo = NULL;
	GdkDisplay*		 pGdkDisplay = NULL;
	Display*		 pDisplay = NULL;
	gint			 iFormatIndex = 0;

	/* sanity check */
	g_assert (pWidget != NULL);

	formatTemplate.doublebuffer = 1;
	ulMask |= GLITZ_FORMAT_DOUBLEBUFFER_MASK;

	pGdkDisplay = gtk_widget_get_display (pWidget);
	if (!pGdkDisplay)
	{
		g_print ("get_glitz_drawable_format(): ");
		g_print ("Could not get gdk-display from widget!\n");
		return NULL;
	}

	pDisplay = gdk_x11_display_get_xdisplay (pGdkDisplay);
	if (!pDisplay)
	{
		g_print ("get_glitz_drawable_format(): ");
		g_print ("Could not get X11-display form gdk-display!\n");
		return NULL;
	}

	iFormatIndex = 0;
	do
	{
		pCurrentFormat = glitz_glx_find_window_format (pDisplay,
							       iScreen,
							       ulMask,
							       &formatTemplate,
							       iFormatIndex++);
		if (pCurrentFormat)
		{
			pVisualInfo = glitz_glx_get_visual_info_from_format
				(pDisplay,
				 iScreen,
				 pCurrentFormat);

			if (pVisualInfo->depth == 32)
			{
				pResultFormat = pCurrentFormat;
				break;
			}
			else
				if (!pResultFormat)
					pResultFormat = pCurrentFormat;
		}
	} while (pCurrentFormat);

	if (!pResultFormat)
	{
		g_print ("get_glitz_drawable_format(): ");
		g_print ("No double buffered GLX-visual found!\n");
		return NULL;
	}

	return pResultFormat;
}

gboolean
set_glitz_colormap (GtkWidget*		     pWidget,
		    gint		     iScreen,
		    glitz_drawable_format_t* pDrawableFormat)
{
	XVisualInfo* pVisualInfo = NULL;
	GdkVisual*   pGdkVisual = NULL;
	GdkColormap* pGdkColormap = NULL;
	GdkDisplay*  pGdkDisplay = NULL;
	Display*     pDisplay = NULL;

	/* sanity check */
	g_assert (pWidget != NULL);

	pGdkDisplay = gtk_widget_get_display (pWidget);
	if (!pGdkDisplay)
	{
		g_print ("set_glitz_colormap(): ");
		g_print ("Could not get gdk-display from widget!\n");
		return FALSE;
	}

	pDisplay = gdk_x11_display_get_xdisplay (pGdkDisplay);
	if (!pDisplay)
	{
		g_print ("set_glitz_colormap(): ");
		g_print ("Could not get X11-display from gdk!\n");
		return FALSE;
	}

	pVisualInfo = glitz_glx_get_visual_info_from_format (pDisplay,
							     iScreen,
							     pDrawableFormat);
	if (!pVisualInfo)
	{
		g_print ("set_glitz_colormap(): ");
		g_print ("Could not visual-info for GLX-visual!\n");
		return FALSE;
	}

	pGdkVisual = gdkx_visual_get (pVisualInfo->visualid);
	if (!pGdkVisual)
	{
		g_print ("set_glitz_colormap(): ");
		g_print ("Could not gdk-visual form X11-visual!\n");
		return FALSE;
	}

	pGdkColormap = gdk_colormap_new (pGdkVisual, TRUE);
	if (!pGdkColormap)
	{
		g_print ("set_glitz_colormap(): ");
		g_print ("Could not gdk-colormap form gdk-visual!\n");
		return FALSE;
	}

	gtk_widget_set_colormap (pWidget, pGdkColormap);

	return TRUE;
}

glitz_drawable_t*
create_glitz_drawable (GtkWidget*		pWidget,
		       glitz_drawable_format_t*	pDrawableFormat,
		       gint			iWidth,
		       gint			iHeight)
{
	glitz_drawable_t* pDrawable = NULL;
	GdkDisplay* pGdkDisplay = NULL;
	Display* pDisplay = NULL;
	Window windowId = 0;

	/* sanity check */
	g_assert (pWidget != NULL);

	/* ??? pGdkDisplay = gtk_widget_get_display (pWidget); */
	pGdkDisplay = gdk_display_get_default ();
	if (!pGdkDisplay)
	{
		g_print ("create_glitz_drawable(): ");
		g_print ("Could not get the default gdk-display!\n");
		return NULL;
	}

	pDisplay = gdk_x11_display_get_xdisplay (pGdkDisplay);
	if (!pDisplay)
	{
		g_print ("create_glitz_drawable(): ");
		g_print ("Could not get X11-display from gdk!\n");
		return NULL;
	}

	windowId = gdk_x11_drawable_get_xid (GDK_DRAWABLE (pWidget->window));
	if (windowId == 0)
	{
		g_print ("create_glitz_drawable(): ");
		g_print ("Could not get id of X11-window from gtk-widget!\n");
		return NULL;
	}

	pDrawable = glitz_glx_create_drawable_for_window (pDisplay,
							  0,
							  pDrawableFormat,
							  windowId,
							  iWidth,
							  iHeight);
	if (!pDrawable)
	{
		g_print ("create_glitz_drawable(): ");
		g_print ("Failed to create glitz-drawable!\n");
		return NULL;
	}

	return pDrawable;
}

glitz_format_t*
get_glitz_format (glitz_drawable_format_t* pDrawableFormat,
		  glitz_drawable_t*	   pDrawable)
{
	glitz_format_t* pFormat = NULL;
	glitz_format_t formatTemplate;

	/* sanity checks */
	g_assert (pDrawableFormat != NULL);
	g_assert (pDrawable != NULL);

	formatTemplate.color = pDrawableFormat->color;
	formatTemplate.color.fourcc = GLITZ_FOURCC_RGB;

	pFormat = glitz_find_format (pDrawable,
				     GLITZ_FORMAT_RED_SIZE_MASK   |
				     GLITZ_FORMAT_GREEN_SIZE_MASK |
				     GLITZ_FORMAT_BLUE_SIZE_MASK  |
				     GLITZ_FORMAT_ALPHA_SIZE_MASK |
				     GLITZ_FORMAT_FOURCC_MASK,
				     &formatTemplate,
				     0);

	if (!pFormat)
	{
		g_print ("get_glitz_format(): ");
		g_print ("Could not find glitz-surface-format!\n");
		return NULL;
	}

	return pFormat;
}

cairo_t*
create_glitz_context (GdkWindow*	       pWindow,
		      glitz_drawable_format_t* pDrawableFormat,
		      glitz_drawable_t*	       pDrawable,
		      glitz_format_t*	       pFormat,
		      gint		       iWidth,
		      gint		       iHeight)
{
	glitz_surface_t* pGlitzSurface = NULL;
	cairo_surface_t* pCairoSurface = NULL;
	cairo_t*	 pContext = NULL;

	pGlitzSurface = glitz_surface_create (pDrawable,
					      pFormat,
					      iWidth,
					      iHeight,
					      0,
					      NULL);

	if (pDrawableFormat->doublebuffer)
		glitz_surface_attach (pGlitzSurface,
				      pDrawable,
				      GLITZ_DRAWABLE_BUFFER_BACK_COLOR);
	else
		glitz_surface_attach (pGlitzSurface,
				      pDrawable,
				      GLITZ_DRAWABLE_BUFFER_FRONT_COLOR);

	pCairoSurface = cairo_glitz_surface_create (pGlitzSurface);
	pContext = cairo_create (pCairoSurface);

	cairo_surface_destroy (pCairoSurface);
	glitz_surface_destroy (pGlitzSurface);

	return pContext;
}
