/*******************************************************************************
**3456789 123456789 123456789 123456789 123456789 123456789 123456789 123456789
**      10        20        30        40        50        60        70        80
**
** project:
**    "Bling it up - make it sexy!"
**
** file:
**    main.c
**
** author:
**    Mirco "MacSlow" Mueller <macslow@bangang.de>
**
** copyright (C) Mirco Mueller, february 2007, placed under the terms of the GPL
**
*******************************************************************************/

#include <gtk/gtk.h>
#include <librsvg/rsvg.h>
#include <librsvg/rsvg-cairo.h>

#include "event-handler.h"
#include "glitz-helper.h"

#define DEFAULT_WIDTH  480
#define DEFAULT_HEIGHT 700

/* pure cerebral mayhem */
GTimer*			 g_pTimer	   = NULL;
gint			 g_iRefreshRate	   = 0;
gboolean		 g_bUseGlitz	   = FALSE;
glitz_drawable_format_t* g_pDrawableFormat = NULL;
glitz_drawable_t*	 g_pDrawable	   = NULL;
glitz_format_t*		 g_pFormat	   = NULL;

int
main (int    argc,
      char** argv)
{
	GtkWidget*	pWindow		= NULL;
	GError*		pError		= NULL;
	guint		uiDrawHandlerId = 0;
	GOptionContext*	pOptionContext	= NULL;
	GOptionEntry	aOptions[]	= {{"refresh-rate",
					    'r',
					    0,
					    G_OPTION_ARG_INT,
					    &g_iRefreshRate,
					    "render at FPS (default: 30 Hz)",
					    "FPS"},
					   {"use-glitz",
					    'g',
					    0,
					    G_OPTION_ARG_NONE,
					    &g_bUseGlitz,
					    "use glitz (default: don't use glitz)",
					    NULL},
					   {NULL}};

	/* setup gtk+ and rsvg */
	gtk_init (&argc, &argv);
	rsvg_init ();

	/* setup and process command-line options */
	pOptionContext = g_option_context_new ("- example for FOSDEM2007 talk");
	g_option_context_add_main_entries (pOptionContext,
					   aOptions,
					   "bling-it-up");
	g_option_context_parse (pOptionContext, &argc, &argv, NULL);
	g_option_context_free (pOptionContext);

	/* create window and set a few window-properties */
	pWindow = gtk_window_new (GTK_WINDOW_TOPLEVEL);
	gtk_window_set_icon_from_file (GTK_WINDOW (pWindow),
				       "./pixmaps/cairo-scarab.png",
				       &pError);
	gtk_widget_set_size_request (pWindow, DEFAULT_WIDTH, DEFAULT_HEIGHT);
	gtk_window_set_resizable (GTK_WINDOW (pWindow), FALSE);
	gtk_widget_set_app_paintable (pWindow, TRUE);
	gtk_window_set_title (GTK_WINDOW (pWindow), "Bling it up!");
	gtk_window_set_decorated (GTK_WINDOW (pWindow), TRUE);

	if (g_bUseGlitz)
	{
		g_pDrawableFormat = get_glitz_drawable_format (pWindow, 0);
		set_glitz_colormap (pWindow, 0, g_pDrawableFormat);
		gtk_widget_set_double_buffered (pWindow, FALSE);
	}

	gtk_widget_show (pWindow);
	gtk_widget_add_events (pWindow,
			       GDK_POINTER_MOTION_MASK |
			       GDK_BUTTON_PRESS_MASK |
			       GDK_BUTTON_RELEASE_MASK);

	/* force a refresh-rate of iRefreshRate Hz and avoid a division by 0 */
	if (g_iRefreshRate <= 0)
		g_iRefreshRate = 30;

	uiDrawHandlerId = g_timeout_add (1000 / g_iRefreshRate,
					 (GSourceFunc) draw_handler, 
					 pWindow);

	/* hook up event-callbacks for keystrokes, close-button, configure and
	** expose/redraw */
	g_signal_connect (G_OBJECT (pWindow),
			  "delete-event",
			  G_CALLBACK (fake_delete_handler),
			  NULL);
	g_signal_connect (G_OBJECT (pWindow),
			  "configure-event",
			  G_CALLBACK (configure_handler),
			  NULL);
	g_signal_connect (G_OBJECT (pWindow),
			  "expose-event",
			  G_CALLBACK (expose_handler),
			  NULL);
	g_signal_connect (G_OBJECT (pWindow),
			  "key-press-event",
			  G_CALLBACK (key_press_handler),
			  NULL);
	g_signal_connect (G_OBJECT (pWindow),
			  "button-press-event",
			  G_CALLBACK (button_handler),
			  NULL);
	g_signal_connect (G_OBJECT (pWindow),
			  "button-release-event",
			  G_CALLBACK (button_handler),
			  NULL);
	g_signal_connect (G_OBJECT (pWindow),
			  "motion-notify-event",
			  G_CALLBACK (motion_handler),
			  NULL);

	if (g_bUseGlitz)
	{
		g_pDrawable = create_glitz_drawable (pWindow,
						     g_pDrawableFormat,
						     DEFAULT_WIDTH,
						     DEFAULT_HEIGHT);
		g_pFormat = get_glitz_format (g_pDrawableFormat, g_pDrawable);
	}

	/* create and start a time, needed for the clock-hands animations */
	g_pTimer = g_timer_new ();

	/* enter event-loop */
	gtk_main ();

	/* cleanup */
	rsvg_term ();
	g_timer_destroy (g_pTimer);

	return 0;
}
